@@ -1,7 +1,5 @@
-use std::env::args;
 use std::sync::{Arc, Mutex};
 
-use gdk::prelude::*;
 use gio::prelude::*;
 #[cfg(all(feature = "notify", not(target_env = "musl")))]
 use glib::clone;
@@ -31,8 +29,7 @@ const APP_TITLE: &str = "prs quick copy";
 const CLIPBOARD_TIMEOUT: u32 = 20;
 
 fn main() {
-    let application =
-        gtk::Application::new(Some(APP_ID), Default::default()).expect("Initialization failed...");
+    let application = gtk::Application::new(Some(APP_ID), Default::default());
     application.connect_activate(|app| {
         build_ui(app);
     });
@@ -47,7 +44,7 @@ fn main() {
     application.add_action(&quit);
 
     // Run the application
-    application.run(&args().collect::<Vec<_>>());
+    application.run();
 }
 
 /// Wraps a store secret.
@@ -70,18 +67,17 @@ impl From<Secret> for Data {
 /// Create GTK list model for given secrets.
 fn create_list_model(secrets: Vec<Secret>) -> gtk::ListStore {
     let data: Vec<Data> = secrets.into_iter().map(|s| s.into()).collect();
-    let col_types: [glib::Type; 1] = [glib::Type::String];
+    let col_types: [glib::Type; 1] = [glib::Type::STRING];
     let store = gtk::ListStore::new(&col_types);
-    let col_indices: [u32; 1] = [0];
     for d in data.iter() {
-        let values: [&dyn ToValue; 1] = [&d.name()];
-        store.set(&store.append(), &col_indices, &values);
+        let values: [(u32, &dyn ToValue); 1] = [(0, &d.name())];
+        store.set(&store.append(), &values);
     }
     store
 }
 
 fn build_ui(application: &gtk::Application) {
-    // Load secrets from store
+    // Load store
     let store = match Store::open(prs_lib::STORE_DEFAULT_ROOT) {
         Ok(store) => store,
         Err(err) => {
@@ -93,6 +89,19 @@ fn build_ui(application: &gtk::Application) {
             return;
         }
     };
+    #[cfg(all(feature = "tomb", target_os = "linux"))]
+    let tomb = store.tomb(false, false, true);
+
+    // Prepare tomb
+    #[cfg(all(feature = "tomb", target_os = "linux"))]
+    if let Err(err) = tomb.prepare() {
+        eprintln!("{}", err);
+        error_dialog("Failed to prepare password store Tomb", None);
+        application.quit();
+        return;
+    }
+
+    // Find secrets
     let secrets = store.secrets(None);
 
     // Quit if user has no secrets
@@ -120,7 +129,7 @@ fn build_ui(application: &gtk::Application) {
     completion.set_inline_completion(true);
     completion.set_inline_selection(true);
     completion.set_match_func(|completion, query, iter| {
-        model_item_text(&completion.get_model().unwrap(), iter)
+        model_item_text(&completion.model().unwrap(), iter)
             .map(|text| text.contains(query))
             .unwrap_or(false)
     });
@@ -146,7 +155,7 @@ fn build_ui(application: &gtk::Application) {
     input_field.connect_activate(move |entry| {
         selected_entry(
             store.clone(),
-            entry.get_text().into(),
+            entry.text().into(),
             window_ref.clone(),
             input_ref.clone(),
         );
@@ -157,6 +166,8 @@ fn build_ui(application: &gtk::Application) {
     // show everything
     window.show_all();
     window.grab_focus();
+
+    // TODO: finalize store tomb somewhere
 }
 
 /// Called when we've selected a secret in the input field.
@@ -239,7 +250,7 @@ fn selected(secret: Secret, window: gtk::ApplicationWindow, input: gtk::SearchEn
     // Hack to unfocus and move window to back
     window.set_accept_focus(false);
     window.set_focus(None::<&gtk::Widget>);
-    if let Some(window) = window.get_window() {
+    if let Some(window) = window.window() {
         window.hide();
         window.show_unraised();
         window.lower();
@@ -341,7 +352,7 @@ fn error_dialog(msg: &str, window: Option<&gtk::ApplicationWindow>) {
 
 /// Get the text for a tree model item by iterator.
 fn model_item_text(model: &gtk::TreeModel, iter: &gtk::TreeIter) -> Option<String> {
-    let item = model.get_value(iter, 0);
+    let item = model.value(iter, 0);
 
     // Get item text
     let text: Result<Option<String>, _> = item.get();
